import 'package:arch/src/initializer.dart';
import 'package:arch/src/internal/service_locator_impl.dart';
import 'package:arch/src/internal/service_registry_impl.dart';
import 'package:arch/src/service_locator.dart';
import 'package:flutter/widgets.dart';

/// The main application.
///
/// Use this to build your [MaterialApp].
abstract class Application extends StatelessWidget {
  Application(Initializer initializer, {Key key})
      : assert(initializer != null),
        super(key: key) {
    initializer.registerTypes(ServiceRegistryImpl()..registerInternalDependencies());
    onInitialize();
  }

  /// The singleton [ServiceLocator] of the current [Application].
  ///
  /// You can begin resolving dependencies after [Initializer.registerTypes], or using [onInitialize].
  ///
  /// This is also used by the [NavigationService] and [DialogService] for view/dialog resolution.
  static final ServiceLocator serviceLocator = ServiceLocatorImpl();

  /// Called after [Initializer.registerTypes] finished registering the types for resolution.
  void onInitialize() {}
}
