import 'package:arch/src/application.dart';
import 'package:arch/src/dialog.dart';
import 'package:arch/src/navigation.dart';
import 'package:arch/src/view.dart';
import 'package:flutter/material.dart';

class DialogServiceImpl implements DialogService {
  @override
  Future<void> alert(String message, {String title, String ok = 'Ok'}) {
    return showDialog(
      context: _context,
      builder: (context) {
        return AlertDialog(
          title: title != null ? Text(title) : null,
          content: Text(message),
          actions: <Widget>[
            FlatButton(
              child: Text(ok),
              onPressed: Navigator.of(context).pop,
            )
          ],
        );
      },
    );
  }

  @override
  Future<bool> confirm(String message, {String title, String ok = 'Ok', String cancel = 'Cancel'}) {
    return showDialog<bool>(
      context: _context,
      builder: (context) {
        return AlertDialog(
          title: title != null ? Text(title) : null,
          content: Text(message),
          actions: <Widget>[
            FlatButton(
              child: Text(cancel),
              onPressed: () => Navigator.of(context).pop(false),
            ),
            FlatButton(
              child: Text(ok),
              onPressed: () => Navigator.of(context).pop(true),
            ),
          ],
        );
      },
    );
  }

  @override
  Future<Map<String, Object>> showCustomDialog(String dialogName, [Map<String, Object> parameters]) {
    return showDialog(
      context: _context,
      builder: (context) => DialogRoute(
        child: _getDialog(dialogName),
        settings: RouteSettings(name: dialogName, arguments: parameters),
      ),
    );
  }

  @override
  void pop([Map<String, Object> result]) => Navigator.of(_context).pop(result);

  static BuildContext get _context {
    assert(NavigationService.navigatorKey.currentState != null,
        'NavigatorState not found for key `NavigationService.navigatorKey`. Assign `NavigationService.navigatorKey` as your app [MaterialApp.navigatorKey].');

    return NavigationService.navigatorKey.currentState.overlay.context;
  }

  static Widget _getDialog(String name) {
    assert(Application.serviceLocator.canResolve<DialogBase>(name: name),
        'Cannot find dialog named $name. Did you registered it?');
    return Application.serviceLocator.resolve<DialogBase>(name);
  }
}
