import 'package:arch/src/application.dart';
import 'package:arch/src/navigation.dart';
import 'package:flutter/material.dart';
import 'package:page_transition/page_transition.dart';

class NavigationServiceImpl implements NavigationService {
  @override
  void pop([Map<String, Object> result]) => _navigator.pop(result);

  @override
  void popUntil(String viewName) {
    assert(viewName != null);
    _navigator.popUntil(ModalRoute.withName('/$viewName'));
  }

  @override
  Future<Map<String, Object>> push(String viewName, [Map<String, Object> parameters]) {
    assert(viewName != null);
    return _navigator.push(
      MaterialPageRoute(
        builder: (_) => _getView(viewName),
        settings: RouteSettings(name: '/$viewName', arguments: parameters),
      ),
    );
  }

  @override
  Future<Map<String, Object>> pushAndRemoveUntil(
    String viewName,
    String removeUntil, [
    Map<String, Object> parameters,
  ]) {
    assert(viewName != null);
    assert(removeUntil != null);
    return _navigator.pushAndRemoveUntil(
      MaterialPageRoute(
        builder: (_) => _getView(viewName),
        settings: RouteSettings(name: '/$viewName', arguments: parameters),
      ),
      ModalRoute.withName('/$removeUntil'),
    );
  }

  @override
  void pushToNewRoot(String viewName, [Map<String, Object> parameters]) {
    assert(viewName != null);
    _navigator.pushAndRemoveUntil(
      PageTransition<void>(
        type: PageTransitionType.fade,
        curve: Curves.easeOutSine,
        duration: Duration(milliseconds: 150),
        child: _getView(viewName),
        settings: RouteSettings(name: '/$viewName', arguments: parameters),
      ),
      (_) => false,
    );
  }

  @override
  Future<Map<String, Object>> pushModal(String viewName, [Map<String, Object> parameters]) {
    assert(viewName != null);
    return _navigator.push(
      MaterialPageRoute(
        builder: (_) => _getView(viewName),
        fullscreenDialog: true,
        settings: RouteSettings(name: '/$viewName', arguments: parameters),
      ),
    );
  }

  @override
  Future<Map<String, Object>> pushReplacement(String viewName, [Map<String, Object> parameters]) {
    assert(viewName != null);
    return _navigator.pushReplacement(
      MaterialPageRoute(
        builder: (_) => _getView(viewName),
        settings: RouteSettings(name: '/$viewName', arguments: parameters),
      ),
    );
  }

  static NavigatorState get _navigator {
    assert(NavigationService.navigatorKey.currentState != null,
        'NavigatorState not found for key `NavigationService.navigatorKey`. Assign `NavigationService.navigatorKey` as your app [MaterialApp.navigatorKey].');

    return NavigationService.navigatorKey.currentState;
  }

  static Widget _getView(String name) {
    assert(Application.serviceLocator.canResolve<Widget>(name: name),
        'Cannot find view/page named $name. Did you registered it?');
    return Application.serviceLocator.resolve<Widget>(name);
  }
}
