import 'package:arch/src/dialog.dart';
import 'package:arch/src/internal/dialog_impl.dart';
import 'package:arch/src/internal/navigation_impl.dart';
import 'package:arch/src/internal/service_locator_impl.dart';
import 'package:arch/src/navigation.dart';
import 'package:arch/src/service_registry.dart';
import 'package:arch/src/view.dart';
import 'package:arch/src/view_model.dart';
import 'package:flutter/material.dart';
import 'package:meta/meta.dart';

class ServiceRegistryImpl extends ServiceLocatorImpl implements ServiceRegistry {
  void registerInternalDependencies() {
    ServiceLocatorImpl.i
      ..registerLazySingleton<NavigationService>(() => NavigationServiceImpl())
      ..registerLazySingleton<DialogService>(() => DialogServiceImpl());
  }

  @override
  void registerFactory<T>(T Function() factory, {String name}) {
    assert(factory != null);
    ServiceLocatorImpl.i.registerFactory(factory, instanceName: name);
  }

  @override
  void registerLazySingleton<T>(T Function() factory, {String name}) {
    assert(factory != null);
    ServiceLocatorImpl.i.registerLazySingleton(factory, instanceName: name);
  }

  @override
  void registerSingleton<T>(T instance, {String name}) {
    assert(instance != null);
    ServiceLocatorImpl.i.registerSingleton(instance, instanceName: name);
  }

  @override
  void registerView<TView extends Widget, TViewModel extends ChangeNotifier>({
    @required TView Function() view,
    @required TViewModel Function() viewModel,
    String viewName,
  }) {
    assert(view != null);
    assert(viewModel != null);
    ServiceLocatorImpl.i
      ..registerFactory<TView>(view, instanceName: viewName ?? TView.toString())
      ..registerFactory<TViewModel>(viewModel);
  }

  @override
  void registerDialog<TDialog extends DialogBase<DialogModelBase>, TDialogModel extends DialogModelBase>({
    @required TDialog Function() dialog,
    @required TDialogModel Function() dialogModel,
    String dialogName,
  }) {
    assert(dialog != null);
    assert(dialogModel != null);
    ServiceLocatorImpl.i
      ..registerFactory<TDialog>(dialog, instanceName: dialogName ?? TDialog.toString())
      ..registerFactory<TDialogModel>(dialogModel);
  }
}
