import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';

/// A widget that builds it self when the [ChangeNotifier] notifies changes.
///
/// Automatically calls [ChangeNotifier.dispose] when this widget is removed from the tree.
class ChangeNotifierBuilder<T extends ChangeNotifier> extends StatelessWidget {
  ChangeNotifierBuilder({
    @required this.create,
    @required this.builder,
    this.child,
    Key key,
  })  : assert(create != null),
        assert(builder != null),
        super(key: key);

  /// Function that creates [T].
  ///
  /// This is called exactly once.
  final T Function() create;

  /// A builder that is called whenever [T] changes.
  final Widget Function(BuildContext, T, Widget) builder;

  /// An optional child to passed to [builder].
  final Widget child;

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider<T>(
      create: (_) => create(),
      child: Consumer<T>(
        builder: (context, model, child) => builder(context, model, child),
        child: child,
      ),
    );
  }
}
